from fabriq.tools.clinical_trials import ClinicalTrialsTool
from fabriq.tools.ner import NER
from fabriq.tools.text_classification import TextClassification
from fabriq.tools.text_summarization import TextSummarization
from fabriq.tools.travel_tool import TravelTool
from fabriq.tools.web_search import WebSearchTool
from fabriq.tools.wiki import WikipediaTool
from fabriq.tools.wolfram import WolframAlphaTool
from fabriq.tools.yahoo_finance_news import YFinanceNews
from fabriq.tools.youtube import YoutubeSearch